package com.magnatune.eyecreate.companionformagnatune.fragments;

import android.content.res.ColorStateList;
import android.graphics.Color;
import android.graphics.drawable.BitmapDrawable;
import android.os.Build;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.app.Fragment;
import android.support.v7.graphics.Palette;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.GridLayout;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.TextView;

import com.magnatune.eyecreate.companionformagnatune.R;
import com.magnatune.eyecreate.companionformagnatune.adapters.TracksAdapter;
import com.magnatune.eyecreate.companionformagnatune.model.Album;
import com.magnatune.eyecreate.companionformagnatune.model.MagnatuneDBManager;
import com.magnatune.eyecreate.companionformagnatune.model.Track;
import com.squareup.picasso.Callback;
import com.squareup.picasso.Picasso;

import io.realm.Realm;

public class AlbumFragment extends Fragment {

    public static final String ARG_ALBUM_ID = "albumsku";

    Realm db;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        db = MagnatuneDBManager.getAlbumDB();
        if(!getArguments().containsKey(ARG_ALBUM_ID)) {
            //No album argument means can't make fragment
            getActivity().onBackPressed();
        }
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        db.close();
    }

    private static void setListViewHeightBasedOnChildren(ListView listView) {
        ListAdapter listAdapter = listView.getAdapter();
        if (listAdapter == null)
            return;

        int desiredWidth = View.MeasureSpec.makeMeasureSpec(listView.getWidth(), View.MeasureSpec.UNSPECIFIED);
        int totalHeight = 0;
        View view = null;
        for (int i = 0; i < listAdapter.getCount(); i++) {
            view = listAdapter.getView(i, view, listView);
            if (i == 0)
                view.setLayoutParams(new ViewGroup.LayoutParams(desiredWidth, GridLayout.LayoutParams.WRAP_CONTENT));

            view.measure(desiredWidth, View.MeasureSpec.UNSPECIFIED);
            totalHeight += view.getMeasuredHeight();
        }
        ViewGroup.LayoutParams params = listView.getLayoutParams();
        params.height = totalHeight + (listView.getDividerHeight() * (listAdapter.getCount() - 1));
        listView.setLayoutParams(params);
    }

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        final View v = inflater.inflate(R.layout.fragment_album, container, false);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            v.findViewById(R.id.album_cover).setTransitionName(getArguments().getString(ARG_ALBUM_ID));
        }
        ListView listView = (ListView) v.findViewById(R.id.album_tracks);
        Album currentAlbum = db.where(Album.class).equalTo("albumsku",getArguments().getString(ARG_ALBUM_ID)).findFirst();
        ((TextView)v.findViewById(R.id.album_name)).setText(currentAlbum.getAlbumname());
        ((TextView)v.findViewById(R.id.artist_name)).setText(currentAlbum.getArtist().getArtistname());
        Picasso.with(getActivity()).load(currentAlbum.getCover_small()).into((ImageView) v.findViewById(R.id.album_cover), new Callback() {
            @Override
            public void onSuccess() {
                Palette colors = Palette.from(((BitmapDrawable) ((ImageView) v.findViewById(R.id.album_cover)).getDrawable()).getBitmap()).generate();
                Palette.Swatch swatch = null;
                swatch = colors.getVibrantSwatch();
                if (swatch == null) {
                    swatch = colors.getMutedSwatch();
                }
                if (swatch == null && colors.getSwatches().size() > 0) {
                    swatch = colors.getSwatches().get(0);
                }
                if (swatch != null) {
                    ((TextView) v.findViewById(R.id.album_name)).setTextColor(swatch.getTitleTextColor());

                    //Get a slightly off color for button background.
                    float[] modColor = swatch.getHsl();
                    modColor[2] = (float) (modColor[2]<=0.5?modColor[2]+0.1:modColor[2]-0.1);
                    ((FloatingActionButton)v.findViewById(R.id.queue_album)).setBackgroundTintList(ColorStateList.valueOf(Color.HSVToColor(modColor)));

                    ((TextView) v.findViewById(R.id.artist_name)).setTextColor(swatch.getBodyTextColor());
                    v.findViewById(R.id.album_details).setBackgroundColor(swatch.getRgb());
                }
            }

            @Override
            public void onError() {

            }
        });
        TracksAdapter adapter = new TracksAdapter(getActivity(),db.where(Track.class).equalTo("albumname",currentAlbum.getAlbumname()).equalTo("artist", currentAlbum.getArtist().getArtistname()).findAllSorted("tracknumber"),true);
        listView.setAdapter(adapter);
        setListViewHeightBasedOnChildren(listView);
        return v;
    }
}
